<?php

namespace Drupal\alternative_user_emails\Plugin\Validation\Constraint;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Validates that a field is unique for the given entity type.
 */
class AlternativeUserEmailsValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($items, Constraint $constraint) {
    if (!$item = $items->first()) {
      return;
    }

    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $items->getEntity();
    if ($entity->getEntityTypeId() !== 'user' || $items->getFieldDefinition()->getName() !== 'alternative_user_emails') {
      return;
    }

    $id_key = $entity->getEntityType()->getKey('id');
    $entity_id = $entity->id();

    // Validate each alternative email given.
    foreach ($items as $item) {
      // Query for other users with the same email.
      $query = \Drupal::entityQuery('user');

      // Exclude the current user.
      // Using isset() instead of !empty() as 0 and '0' are valid ID values for
      // entity types using string IDs.
      if (isset($entity_id)) {
        $query->condition($id_key, $entity_id, '<>');
      }

      // Check the primary and alternative emails on other users.
      $or = $query->orConditionGroup()
        ->condition('mail', $item->value)
        ->condition('alternative_user_emails', $item->value);
      $query->condition($or);

      $value_taken = (bool) $query
        ->range(0, 1)
        ->count()
        ->execute();

      if ($value_taken) {
        $this->context->addViolation($constraint->message, [
          '%value' => $item->value,
        ]);
      }
    }

  }

}
